$.extend({
    /**
    * Загружает контент в диалоговом окне. Зависит от jquery.colorbox, jquery.form
    */
    openDialog: function(options) {
        
        options = $.extend({
            url: '',
            width: null, 
            data: null, 
            callback: null,
            colorboxOptions: {},
            bindSubmit:true
        }, options);
        
        var colorBoxParams = $.extend(
            { //Отображаем диалог в режиме ожидания данных
                open:true, 
                width: (options.width) ? options.width : false,
                opacity: 0.4,
                onClosed: function() {
                    xhr.abort();
                }
            }, options.colorboxOptions);        
        
        var onComplete = function(response) {
            if (options.bindSubmit) {
                $('#cboxLoadedContent').each(function() {
                    var content = this;
                    $(content).trigger('new-content');
                    $('form', this).each(function() 
                    {
                        var new_action = $(this).attr('data-ajax-action');
                        if (typeof(new_action) != 'undefined') {
                            $(this).attr('action', new_action);
                        }
                        $(this).ajaxForm({
                            dataType: 'json',
                            data: {dialogWrap: 1},
                            beforeSubmit: function(arr, form, options) {
                                $('button[type="submit"], input[type="submit"]',form).attr('disabled','disabled'); //Защита от двойного клика
                            },
                            success: function(response) {
                                if (response.closeDialog) {
                                    $.colorbox.close();
                                }
                                if (response.reloadPage) {
                                    location.reload();
                                    return;
                                }
                                
                                if (!response.closeDialog) {
                                    $(content).empty();
                                    var html = $('<div>').hide().appendTo('body').append(response.html);
                                    $.colorbox($.extend({html:html.show()}, colorBoxParams));
                                    onComplete();
                                }
                            }
                        });
                        
                    });
                });
            }
            if (options.callback) options.callback(response);
        };
        
        var param = options.url.indexOf('?') == -1 ? '?dialogWrap=1' : '&dialogWrap=1';
        var xhr = $.get(options.url + param, options.data, function(response){
            var html = $('<div>').hide().appendTo('body').append(response.html);
            
            var dialogData = html.find('[data-dialog-options]').data('dialogOptions');
            if (dialogData) {
                colorBoxParams = $.extend(colorBoxParams, dialogData);
            }
            $.colorbox($.extend({
                    html: html.show(),
                    onComplete: function() {
                        onComplete(response);
                    }
                }, colorBoxParams)
            );
        }, 'json');        
        
        $.colorbox(colorBoxParams);
        
        return false;        
    },
    
    /**
    * Инициализирует функцию добавления товаров в корзину, отображение общего числа товаров в корзине
    * Инициализируется, если на экране присутствует блок "Корзина". (элемент #cart)
    */
    cart: function( method ) {
        var defaults = {
            cartActiveClass: 'active',
            fixedLine: '.fixedCart',
            fixedLineActiveClass: 'cartIsOpen',
            amountTextField: '.num',
            closeDlg: '.closeDlg',
            popupCartId:'popupCart',
            checkoutButton: '#checkout',
            checkoutButtonActiveClass: 'active',
            cartTotalPrice: '.cost .value',
            cartTotalItems: '.products .value',
            cartPriceBlock: false,
            productWrapper: '.productItem',
            update: '.recalc',
            inLoadingClass: 'inloading',
            couponField: '.couponCode',
            couponError: '.couponError',
            saveScroll: '.viewport',
            addToCart: '.addToCart',
            noShowCart: false,
            applyCoupon: '.applyCoupon',
            cartBlock: '#cartItems',
            cartForm: '#cartForm',
            cartItem: '.cartitem',
            amountWraper: '.amount', //Оборачивающий контейнер
            amountField: '.fieldAmount',
            showCart: '.showCart',
            checkout: '.submit',
            clearCart: '.clearCart',
            offerField: '.offer',
                cartItemInc: '.inc',
                cartItemDec: '.dec',
                cartItemRemove: '.remove',
            removeCoupon: '.linesContainer .remove',
            context: '[data-id]',
            offerFormName: 'offer',
            multiOffersName: 'multioffers[items]',    //атрибут name для многомерных компл.            
            multiofferFormName: '[multioffers][',     //атрибут name многомерной комплектации
            multiofferHidden: '[name^="hidden_offers"]',      //атрибут name у спрятанных полей со сведения о компл.            
            amountFormName: 'amount',
            alreadyInCartClass: 'added',
            alreadyInCartClassTimeout:5,
            alreadyInCartBlock: false,
            alreadyInCartBlockTimeout:5,
            concomitantField: '.fieldConcomitant',
            concomitantAmountClass: 'concomitant', //Класс на вводе количества сопутствующих товаров
            continueButton: '.continue'
        },
        $this = $('#cart'),
        hasCart = true,
        $cartBlock;
        
        if (!$this.length) {
            $this = $('body');
            hasCart = false;
        }
        
        var data = $this.data('cart');
        if (!data) { //Инициализация
            data = { options: defaults };
            $this.data('cart', data);
        }
        
        //public
        var methods = {
            init: function(initoptions) {
                data.options.url = $this.data('compareUrl');
                data.options = $.extend(data.options, initoptions);

                $(data.options.addToCart).live('click.cart', addToCart);
                $(data.options.showCart).click(function() {
                    return methods.add($(this).attr('href'));
                });
                $(data.options.checkoutButton).click(function() {
                    if (!$(this).hasClass(data.options.checkoutButtonActiveClass)) return false;
                });
                initCart(); //Вызываем на случай, если находимся на странице с корзиной
            },
            
            /**
            * Добавляет товар в корзину
            * 
            * @param url - ссылка на добавление товара в корзину
            * @param offer - номер комплектации 0,1,2...
            * @param amount - количество товаров, которое необходимо добавить
            * @param noShowCartDialog - не показывать корзину пользователя
            */
            add: function(url, offer, multioffers, amount, noShowCartDialog) {
                
                
                if ($.detectMedia && ($.detectMedia('mobile') || $.detectMedia('portrait')) ) return true; //Не открываем окна в мобильной версии
                
                var params = {};
                if (offer) params.offer = offer;
                if (amount) params.amount = amount;
                if (multioffers) params = $.extend(params, multioffers);
                
                if (!noShowCartDialog) {
                    methods.showCart(url, params);
                } else {
                    $.post(url, params, function(response) {
                        try {
                            //Обновляем сведения в основном окне, если действие происходит во всплывающем
                            window.opener.jQuery.cart('updateCartBlock', response);
                        } catch (e) {
                            $.cart('updateCartBlock', response);
                        }
                    }, 'json');
                }
                   
                $(this).trigger('product.add', {
                    url:url, 
                    offer:offer, 
                    multioffers:multioffers, 
                    amount:amount, 
                    noShowCartDialog:noShowCartDialog
                });
                
                return false;
            },
            
            updateCartBlock: function(serverData) {
                if (serverData) {
                    $(data.options.cartTotalItems, $this).text(serverData.cart.items_count);
                    $(data.options.cartTotalPrice, $this).text(serverData.cart.total_price);
                    $(data.options.cartPriceBlock).toggle(parseInt(serverData.cart.total_price)>0);
                    $(data.options.checkoutButton).toggleClass(data.options.checkoutButtonActiveClass, serverData.cart.can_checkout && parseInt(serverData.cart.items_count)>0 );
                    $this.toggleClass(data.options.cartActiveClass, parseInt(serverData.cart.items_count)>0);
                }
            },
            
            /**
            * Обновляет корзину
            */
            refresh: function(url, callback) {
                var params = $(data.options.cartForm).serializeArray();
                if (!url) url = $(data.options.cartForm).attr('action');  
                showLoading();
                
                $.ajax({
                    url: url,
                    data: params,
                    type: 'POST',
                    dataType: 'json',
                    success: function(response) {
                        if (response.redirect) {
                            location.href = response.redirect;
                        }
                        
                        var scrollElement = $(data.options.saveScroll, $cartBlock),
                            saveScroll = scrollElement.scrollTop();
                        
                        $cartBlock.replaceWith(response.html);
                        initCart(response);
                        $(data.options.saveScroll, $cartBlock).scrollTop(saveScroll);                        
                        $.colorbox.resize();
                        if (callback) callback();
                    }
                });
            },
            
            showCart: function(url, params) {
                params.floatCart = 1;
                $.ajax({
                    url:url,
                    data:params,
                    dataType:'json',
                    success: function(response) {
                        $('#'+data.options.popupCartId).remove();
                        $('<div id="'+data.options.popupCartId+'" />').hide().append(response.html).appendTo('body').fadeIn();
                        $(data.options.fixedLine).addClass(data.options.fixedLineActiveClass);
                        initCart(response);
                    }
                });
            }
        };
        
        //private
        var addToCart = function() {
            if ($.detectMedia && $.detectMedia('mobile')) return true; //Не открываем окна в мобильной версии
            var _this = this;
            $(this).addClass(data.options.alreadyInCartClass);
            if (!$(this).data('storedText') && $(this).data('addText')) {
                $(this).data('storedText', $(this).html());
                $(this).html($(this).data('addText'));
            }
            
            var reset = function() {
                $(_this).removeClass(data.options.alreadyInCartClass);
                if ($(_this).data('storedText')) {
                    $(_this).html($(_this).data('storedText'));
                    $(_this).data('storedText', null);
                }
            };
            
            if (data.options.alreadyInCartClassTimeout) {
                setTimeout(reset, data.options.alreadyInCartClassTimeout * 1000);
            }
            //Отображает всплывающий блок возле корзины
            if (data.options.alreadyInCartBlock) {
                var $block = $(data.options.alreadyInCartBlock);
                clearTimeout($block.data('timer'));                        
                var closeBlock = function(e) {
                    if (e && $(e.target).is(data.options.addToCart)) return;
                    $block.fadeOut();
                }
                
                $block.fadeIn().click(function(e) {
                    e.stopPropagation();
                });
                $('body').one('click', closeBlock);
                
                $block.data('timer', setTimeout(closeBlock, data.options.alreadyInCartClassTimeout * 1000));
            }            
            
            var context = $(this).closest(data.options.context);
            if ($('[name="'+data.options.offerFormName+'"]:checked', context).length) {
                var offer = $('[name="'+data.options.offerFormName+'"]:checked', context).val();
            } else {
                var offer = $('[name="'+data.options.offerFormName+'"]', context).val();
            }
            var amount = $('[name="'+data.options.amountFormName+'"]', context).val();
            
            //Многомерные комплектациями
            if ($('[name^="multioffers["]',context).length){
               var multioffers = {};
               $('[name^="multioffers["]',context).each(function(i){
                   pid              = $(this).attr('name'); 
                   multioffers[pid] = $(this).val();
               }); 
            }            
            
            var noShowCart = $(this).hasClass('noShowCart') || data.options.noShowCart;
            return methods['add'].call(this, $(this).data('href') ? $(this).data('href') : $(this).attr('href'), (offer) ? offer : 0 ,(multioffers) ? multioffers : false, amount, noShowCart);
        },
        
        initCart = function(serverData) {
            $cartBlock = $(data.options.cartBlock);
            if ($cartBlock.length) {
                $cartBlock
                .on('click', data.options.cartItemInc, incProduct)
                .on('click', data.options.cartItemDec, decProduct)
                .on('blur', data.options.amountField, function() {
                    var val = (+$(this).val());
                    if (val<1) $(this).val(1).keyup();
                })
                .on('focus', data.options.amountField, function(e, noSelect) {
                    if (!noSelect) {
                        this.select();
                    }
                })
                .on('keyup', data.options.amountField, updateAmount)
                .on('change', data.options.concomitantField, updateAmount)
                .on('change', data.options.offerField, function() {
                    methods.refresh();
                })
                .on('change', '[name*="'+data.options.multiofferFormName+'"]', changeMultiOffers)
                .on('click', data.options.continueButton, function() {
                    $(this).closest('#colorbox').length ? $.colorbox.close() : history.back();
                    return false;
                })
                .on('click', data.options.cartItemRemove, removeProduct)
                .on('click', data.options.clearCart, clearCart)
                .on('click', data.options.applyCoupon, applyCoupon)
                .on('click', data.options.checkout, checkout)
                .on('click', data.options.closeDlg, closeDlg)
                .bind('click', function(e) { e.stopPropagation(); });
                
                $('html, body').unbind('.cart').one('click.cart', function() {
                    closeDlg();
                });
                
                $(data.options.cartForm, $cartBlock).submit(function() {
                    clearTimeout($cartBlock.data('changeTimer'));
                    methods.refresh();
                    return false;
                })
                
                $(data.options.update, $cartBlock).hide();
            }
            
            methods.updateCartBlock(serverData);
        },
/**
        * Смена(Выбор) многомерных комплектаций в корзине
        * 
        */
        changeMultiOffers = function() {
            var context    = $(this).closest(data.options.context);    // Оборачивающий контейнер
            var offersList = $(data.options.multiofferHidden,context); // Список скрытых полей со значениями
            
            if (offersList.length>0){ //Если комплектции используются
               var offer   = $('[name*="['+data.options.offerFormName+']"]',context); // Скрытое поле - текущая комплектация 
               var offers_info = [];
               //Соберём информацию о том, что выбрано для товара
               var selected = $('[name*="'+data.options.multiofferFormName+'"]',context);
               
               //Соберём информацию о комплектациях для одного товара
               offersList.each(function(i){
                  offers_info[i]          = {};
                  offers_info[i]['id']    = this;
                  offers_info[i]['info']  = $(this).data('info'); 
                  offers_info[i]['num']   = $(this).data('num'); 
                  offers_info[i]['value'] = $(this).val(); 
               });
               //Соберём информацию, что изменилось
               $(selected).each(function(i){
                  selected[i]          = {};
                  selected[i]['title'] = $(this).data('prop-title');
                  selected[i]['value'] = $(this).val();
               });
               
               var sel_offer = false;
               //Отметим, что мы выбрали
               for(var j=0;j<offers_info.length;j++){
                    var info = offers_info[j]['info']; //Группа с информацией
                    
                    var found = 0;                //Флаг, что найдены все совпадения
                    for(var m=0;m<info.length;m++){
                        for(var i=0;i<selected.length;i++){
                           if ((selected[i]['title']==info[m][0])&&(selected[i]['value']==info[m][1])){
                               found++;
                           } 
                        }
                        if (found==selected.length){ //Если удалось найди совпадение, то выходим
                            sel_offer = offers_info[j];
                            break;
                        }
                    }
               }
               
               $(context).removeClass('notAvaliable');   
               
               if (sel_offer){ //Если комплектация найдена
                  offer.val(sel_offer.value);
               }else{ //Если не найдена комплектация, выберем нулевую
                  offer.val(0); 
               }
            }
                 
            methods.refresh();
        },        
        
        incProduct = function() {
            var amountField = $(this).closest(data.options.amountWraper).find(data.options.amountField);
            var amountTextField = $(this).closest(data.options.amountWraper).find(data.options.amountTextField);
            amountField.val( (+amountField.val())+1 ).keyup();
            amountTextField.text( amountField.val() );
            
            return false;
        },
        
        closeDlg = function() {
            $('#'+data.options.popupCartId).fadeOut();
            $(data.options.fixedLine).removeClass(data.options.fixedLineActiveClass);
        },
        
        decProduct = function() {
            var amountField = $(this).closest(data.options.amountWraper).find(data.options.amountField);
            var amountTextField = $(this).closest(data.options.amountWraper).find(data.options.amountTextField);
            
            if ((+amountField.val())>1) {
                amountField.val( (+amountField.val())-1 ).keyup();
                amountTextField.text( amountField.val() );
            }
            return false;
        },
        
        removeProduct = function() {
            var cartItem = $(this).closest(data.options.cartItem);
            cartItem.css('opacity', 0.5).addClass('removing');
            var other = cartItem.siblings('[data-product-id="'+cartItem.data('productId')+'"]');
            if (!other.length) {
                //Удаляем пометку, что товар добавлен в корзину
                var context = $(data.options.productWrapper+'[data-id="'+cartItem.data('productId')+'"]')
                $(data.options.addToCart, context).removeClass(data.options.alreadyInCartClass);
            }
            
            methods.refresh( $(this).attr('href') );
            return false;
        },
        
        showLoading = function() {
            $cartBlock.addClass(data.options.inLoadingClass);
        },
        
        hideLoading = function() {
            $cartBlock.removeClass(data.options.inLoadingClass);
        },
        
        updateAmount = function(e) {
            if (e.keyCode != 13) {
                var _this = $(this);

                var n = _this.closest(data.options.cartItem).index();
                clearTimeout($cartBlock.data('changeTimer'));
                $cartBlock.data('changeTimer', setTimeout(function() {
                    if (_this.val() != '') {
                        methods.refresh(null, function() {
                            //Восстанавливаем фокус
                            $(data.options.cartItem, $cartBlock).eq(n).find(data.options.amountField).each(function() {
                                $(this).trigger('focus', true);
                                var value = $(this).val();
                                $(this).val('').val(value);
                            })
                        });
                    }
                }, 500));
            }
        },
        
        clearCart = function() {
            var context = $(data.options.productWrapper+'[data-id]')
            $(data.options.addToCart, context).removeClass(data.options.alreadyInCartClass);
            return refreshByHref.call(this);
        },
        
        refreshByHref = function() {
            methods.refresh( $(this).attr('href') );
            return false;
        },
        
        applyCoupon = function() {
            methods.refresh();
        },
        
        checkout = function(e) {
            var url = $(data.options.cartForm).attr('action');
            var param = (url.indexOf('?') > -1 ? '&' : '?') + 'checkout=1';
            methods.refresh(url + param);
            return false
        };
  
        if ( methods[method] ) {
            methods[ method ].apply( this, Array.prototype.slice.call( arguments, 1 ));
        } else if ( typeof method === 'object' || ! method ) {
            return methods.init.apply( this, arguments );
        }       
    }
    

});


$.fn.extend({
        
    /**
    * Инициализирует открытие контента по ссылке во всплывающем окне
    */
    openInDialog: function( method ) {
        var defaults = {},
        args = arguments;
        
        return this.each(function() {
            var $this = $(this), 
                data = $this.data('openInDialog');

            var methods = {
                init: function(initoptions) {
                    if (data) return;
                    data = {}; $this.data('openInDialog', data);
                    data.options = $.extend({}, defaults, initoptions);
                    $this.click(onClick);
                }
            };
            
            //private 
            var onClick = function() {
                if ($.detectMedia && $.detectMedia('mobile')) return true; //Не открываем окна в мобильной версии
                
                $.openDialog({
                    url: $(this).data('href') ?  $(this).data('href') : $(this).attr('href')
                });
                
                return false;
            }
            
            if ( methods[method] ) {
                methods[ method ].apply( this, Array.prototype.slice.call( args, 1 ));
            } else if ( typeof method === 'object' || ! method ) {
                return methods.init.apply( this, args );
            }        
        });        
        
    }    
});
